/*
* [chimei25khash.c]
*
* Copyright (C) 2006 Orkney Inc., Kazuhide Kobayashi
*
* This program is free software; you can redistribute and/or modify
* it under the terms of the GNU Lesser General Public License
* as published by the Free Software Foundation.
*
* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
* GNU Lesser General Public License for more details.
*
* You should have received a copy of the GNU Lesser General Public
* License along with this program; if not, write to the Free Software
* Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307,
* USA.
*
* See the COPYING file for more information.
*
*
*/

/***********************************************************************

	Chimei25kHash Function : 25000nnbVe[u֐

************************************************************************
*/

#ifdef WIN32
#include <windows.h>
#endif

#include <stdio.h>
#include <string.h>
#include <malloc.h>
#include <math.h>

#include "geocoder.h"
#include "gcjlib.h"

/*-------------------------
	ϐ
---------------------------
*/

	/* 25000nnbVe[u */
static CHIMEI25K_HASH_HEADER m_chimei25kHashHeader;
static int *m_chimei25kHashTable;
static CHIMEI25K_HASH_BUCKET *m_aryBucket;

static int m_numRecords;
static int m_hashSize;

/*---------------------------------------------------------------------

	25000nnbVe[ut@C̓eɊi[B
	
	(in)
		fileName : 25000nnbVe[ut@C̖
	
	E͂̊֐ŊmۂB
	ȄIAChimei25kHashEnd()ĂяoŃJB
	
	߂l
		0 : 	I
		F	G[
			-1 : nbVpmۂłȂ
			-2 : t@CI[vłȂ
			-3 : t@CID̕񂪈Ⴄ
			-4 .. -8 : t@C̃[hG[
			
-----------------------------------------------------------------------
*/

int Chimei25kHashLoad( char *fileName )
{
	FILE *fpHash;
	int n,nRet;
	
	fpHash = fopen( fileName, "rb" );
	if ( fpHash == NULL ){ nRet = -2; goto err; }
	
		/* wb_[ */
	n = fread( &m_chimei25kHashHeader, sizeof( m_chimei25kHashHeader ), 1, fpHash);
	if ( n != 1 ){ nRet = -4; goto err; }

	if ( strcmp( m_chimei25kHashHeader.fileId, FILE_ID_CHIMEI25KHASH ) ) { nRet = -3; goto err; }
	m_numRecords = m_chimei25kHashHeader.numRecords;
	m_hashSize = m_chimei25kHashHeader.hashSize;
	
		/* nbVe[u */
	m_chimei25kHashTable = (int*) malloc( m_hashSize * sizeof(int) );
	if ( ! m_chimei25kHashTable ){ nRet = -1; goto err; }
	
	n = fread( m_chimei25kHashTable, sizeof(int), m_hashSize, fpHash);
	if ( n != m_hashSize ){ nRet = -5; goto err; }

		/* oPbg */
	m_aryBucket = (CHIMEI25K_HASH_BUCKET*) malloc( sizeof(CHIMEI25K_HASH_BUCKET) * m_numRecords );
	if ( ! m_aryBucket ){ nRet = -1; goto err; }

	n = fread( m_aryBucket, sizeof(CHIMEI25K_HASH_BUCKET), m_numRecords, fpHash );
	if ( n != m_numRecords ) { nRet = -6; goto err; }
	
	fclose( fpHash );
	return 0;

err:
	if ( m_aryBucket ) {
		free( m_aryBucket );
		m_aryBucket = NULL;
	}
	if ( m_chimei25kHashTable ) {
		free( m_chimei25kHashTable );
		m_chimei25kHashTable = NULL;
	}
	fclose( fpHash );
	return nRet;
}


/*---------------------------------------------------------------------

	25000nnbVe[ȕIs
	
	EmۂJB
	
	߂l
		0 : 	I
		
-----------------------------------------------------------------------
*/

int Chimei25kHashEnd()
{
	if ( m_aryBucket ) {
		free( m_aryBucket );
		m_aryBucket = NULL;
	}
	if ( m_chimei25kHashTable ) {
		free( m_chimei25kHashTable );
		m_chimei25kHashTable = NULL;
	}
	return 0;
}


/*---------------------------------------------------------------------

	25000nnbVe[uB
	
	(in)
		str        : n
		maxArray   : codeArray̍ővf
	(out)
		codeArray : nR[h
		  
	߂l
		 : i[nR[h̐
		   0 : Ȃ
		F G[
			-1 : nbVe[uɃ[hĂȂ
			-2 : n̒O
			-3 : codeArray̍ővf܂Ŋi[B

	EnbVlstr̐̕擪WoCgiSjɂČvZ
	@ĂB񂪂WoCg̏ꍇ͂邾ōsĂB
	@̂߁A񂪂WoCgȏ̏ꍇ́Aʂ̒nR[h
	@̃R[h擾āÃR[hɓĂ閼̂ƈv邩
	@ǂ`FbN邱ƁB
	@
-----------------------------------------------------------------------
*/

int Chimei25kHashSearch( char *str, int *codeArray, int maxArray )
{
	int hash,index;
	long bucketNum;
	char *p;
	unsigned char *ph;
	unsigned int hv;

	if ( ! m_chimei25kHashTable ) return -1;
	if ( ! strlen( str ) ) return -2;
	
	/* nbVľvZ */
	ph = (unsigned char *) str;
	hv = 0;
	while ( *ph ) {
        hv = hv * 3701 + *ph++;
    }
    hash = hv % m_hashSize;

	/* nbVe[uoPbgԍ擾 */
	bucketNum = m_chimei25kHashTable[hash];
	index = 0;	
	
	/* oPbg̒strƈv̂擾 */
	if ( bucketNum ){	/* nbVlv̂ */
		while (1){
			 p = strstr( str, m_aryBucket[bucketNum].name );
			if ( p == str ){	/* ̍ŏłΌ */
				codeArray[index++] = m_aryBucket[bucketNum].chimeiCode;
				if ( index == maxArray ) return -3;
			}
			bucketNum = m_aryBucket[bucketNum].nextBucketNum ;
			if ( bucketNum ) continue;
			else break;
		}
	}
	return index;
}
