/*
* [chohash.c]
*
* Copyright (C) 2006 Orkney Inc., Kazuhide Kobayashi
*
* This program is free software; you can redistribute and/or modify
* it under the terms of the GNU Lesser General Public License
* as published by the Free Software Foundation.
*
* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
* GNU Lesser General Public License for more details.
*
* You should have received a copy of the GNU Lesser General Public
* License along with this program; if not, write to the Free Software
* Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307,
* USA.
*
* See the COPYING file for more information.
*
*
*/

/***********************************************************************

	ChoHash Function : 厚nbVe[u֐

************************************************************************
*/

#ifdef WIN32
#include <windows.h>
#endif

#include <stdio.h>
#include <string.h>
#include <malloc.h>

#include "geocoder.h"
#include "gcjlib.h"

#define TRUE 1
#define FALSE 0

/*-------------------------
	ϐ
---------------------------
*/

	/* 厚nbVe[u */
static CHO_HASH_HEADER m_choHashHeader;
static long 			m_hashTableOffset[ GYOSEI_MAX ];
static long			*m_hashTable;
static CHO_HASH_BUCKET *m_aryBucket;


/*---------------------------------------------------------------------

	厚nbVe[ut@C̓eɊi[B
	
	(in)
		fileName : 厚nbVe[ut@C̖
	
	E͂̊֐ŊmۂB
	ȄIAChoHashEnd()ĂяoŃJB
	
	߂l
		0 : 	I
		F	G[
			-1 : nbVe[upmۂłȂ
			-2 : nbVoPbgpmۂłȂ
			-3 : t@CI[vłȂ
			-4 : t@CID̕񂪈Ⴄ
			-5 .. -8 : t@C̃[hG[
			
-----------------------------------------------------------------------
*/

int ChoHashLoad( char *fileName )
{
	FILE *fpHash;
	int n,numRecords,hashTableBytes,nRet;
	
	fpHash = fopen( fileName, "rb" );
	if ( fpHash == NULL ){ nRet = -3; goto err; }
	
		/* wb_[ */
	n = fread( &m_choHashHeader, sizeof( m_choHashHeader ), 1, fpHash);
	if ( n != 1 ){ nRet = -5; goto err; }

	if ( strcmp( m_choHashHeader.fileId, FILE_ID_CHOHASH ) ) { nRet = -4; goto err; }
	numRecords = m_choHashHeader.numRecords;
	hashTableBytes = m_choHashHeader.hashTableBytes;
	
		/* e[ũItZbgi[z */	
	n = fread( &m_hashTableOffset, sizeof( m_hashTableOffset ), 1, fpHash);
	if ( n != 1 ){ nRet = -6; goto err; }
	
		/* nbVe[u */
	m_hashTable = (long*) malloc( hashTableBytes );
	if ( ! m_hashTable ){ nRet = -1; goto err; }
	
	n = fread( m_hashTable, hashTableBytes, 1, fpHash);
	if ( n != 1 ){ nRet = -7; goto err; }

		/* oPbg */
	m_aryBucket = (CHO_HASH_BUCKET*) malloc( sizeof(CHO_HASH_BUCKET) * numRecords );
	if ( ! m_aryBucket ){ nRet = -2; goto err; }

	n = fread( m_aryBucket, sizeof(CHO_HASH_BUCKET), numRecords, fpHash );
	if ( n != numRecords ) { nRet = -8; goto err; }
	
	fclose( fpHash );
	return 0;

err:
	if ( m_aryBucket ) {
		free( m_aryBucket );
		m_aryBucket = NULL;
	}
	if ( m_hashTable ) {
		free( m_hashTable );
		m_hashTable = NULL;
	}
	fclose( fpHash );
	return nRet;
}


/*---------------------------------------------------------------------

	厚nbVe[ȕIs
	
	EmۂJB
	
	߂l
		0 : 	I
		
-----------------------------------------------------------------------
*/

int ChoHashEnd()
{
	if ( m_aryBucket ) {
		free( m_aryBucket );
		m_aryBucket = NULL;
	}
	if ( m_hashTable ) {
		free( m_hashTable );
		m_hashTable = NULL;
	}
	return 0;
}



/*---------------------------------------------------------------------

	厚nbVe[ugpāAZ̐擪ɂ钬厚
	B
	
	(in)
		cityCode   : s撬R[hiTj
		str        : Z
	(out)
		matchLength : }b`O񕔕̃oCg
		choCode     : 厚̃R[h
		  
	߂l
			 1 : 
			 0 : Ȃ
		F G[
			-1 : nbVe[uɃ[hĂȂ
			-2 : ̒O
			-3 : s撬R[h͈̔̓G[
			-4 : w肳ꂽs撬̃nbVe[u
			-5 : nbVe[u̎s撬R[hȂ
			-6 : z̊mۃG[

	Ew肳ꂽs撬̒厚̂ŁAZ̐擪ɍv̂TB

	E厚͏Z̐擪matchLength̃oCg
	@łB
	@̐̕ȕ\LChoRecGet()֐ɂm鎖łB

-----------------------------------------------------------------------
*/

#define FMAX 300

int ChoHashSearch( int cityCode, char *str, int *matchLength, int *choCode )
{
	int hashSize,hash,foundCode[FMAX][2],maxLength,numFound,i,retry;
	long bucketNum,offset,*hashTable;
	char *p,choBuff[STRMAX+1];

	if ( ! m_hashTable ) return -1;
	if ( ! strlen( str ) ) return -2;
	if ( cityCode < 1000 || cityCode >= GYOSEI_MAX ) return -3;
	
	/* w肳ꂽs撬̃nbVe[üʒu擾 */
	offset = m_hashTableOffset[ cityCode ];
	if ( ! offset ) return -4;	/* w肳ꂽs撬̃nbVe[u */
	
	/* nbVe[us撬R[hŃ`FbN */
	hashTable = m_hashTable + offset;
	if ( *hashTable != cityCode ) return -5;	/* nbVe[u̎s撬R[hȂ */

	hashTable++;
	hashSize = *( hashTable++ );

	/* w蕶̃nbVľvZ */
	
	hash = (*(unsigned long*)(str)) % hashSize;

	retry = 0;
jretry:
	/* nbVe[uoPbgԍ擾 */
	bucketNum = hashTable[hash];

	numFound = 0;	
	/* oPbg̒strƈv̂擾 */
	if ( bucketNum ){	/* Qv̂ */
		while (1){
			p = strstr( str, m_aryBucket[bucketNum].name );
			if ( p ){
				if ( p == str ){	/* ̍ŏłΌ */
					/* }b`ő咷̂̂I邽߂Ɉꎞۑ */
					foundCode[ numFound ][0] = m_aryBucket[bucketNum].choCode;
					foundCode[ numFound++ ][1] = strlen( m_aryBucket[bucketNum].name );
					if ( numFound == FMAX ) return -6;
				}
			}
			bucketNum = m_aryBucket[bucketNum].nextBucketNum ;
			if ( bucketNum ) continue;
			else break;
		}
		if ( ! numFound ) goto j1moji;

		if ( numFound == 1  ) maxLength = foundCode[0][1];
		else {		/* ꍇ͍ő咷̂̂T */
			maxLength = 0;
			for ( i = 0; i < numFound; i++ )
				if ( foundCode[i][1] > maxLength ) maxLength = foundCode[i][1];
		}
		
		/* ő咷̂̂ŁAŏɌꂽ̂Ԃi̔j */
		for ( i = 0; i < numFound; i++ ){
			if ( (foundCode[i][1]) == maxLength ) {
				*choCode = foundCode[i][0];
				*matchLength = maxLength;
				return 1;
			}
		}
		return 0;
	}
	else {	
j1moji:
			/* P̒厚Ȃ */
		if ( retry == 0 ){
			memcpy( choBuff, str, 2 );
			*( choBuff + 2 ) = *( choBuff + 3 ) = (char) 0xff;
			hash = ( *(unsigned long*) choBuff ) % hashSize;
			retry = 1;
			goto jretry;
		}
			/* 1.5̒厚(F1jȂ */
		else if ( retry == 1 ){
			if ( (! IsSjisKanji( str )) && IsSjisKanji( str+1 ) ){
				memcpy( choBuff, str, 3 );
				*( choBuff + 3 ) = '\0';
				hash = ( *(unsigned long*) choBuff ) % hashSize;
				retry = 2;
				goto jretry;
			}
		}
	}
	return 0;
}

