/*
* [jcjadrcode2point.c]
*
* Copyright (C) 2006 Orkney Inc., Kazuhide Kobayashi
*
* This program is free software; you can redistribute and/or modify
* it under the terms of the GNU Lesser General Public License
* as published by the Free Software Foundation.
*
* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
* GNU Lesser General Public License for more details.
*
* You should have received a copy of the GNU Lesser General Public
* License along with this program; if not, write to the Free Software
* Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307,
* USA.
*
* See the COPYING file for more information.
*
*
*/


/***********************************************************************

						Z擾B

************************************************************************
*/

#ifdef WIN32
#include <windows.h>
#endif

#include <stdio.h>
#include <string.h>
#include <malloc.h>

#include "geocoder.h"
#include "gcjlib.h"

static int GetGaikuPoint( ADR_CODE *adrCode , int *secLat, int *secLon );

/*---------------------------------------------------------------------

	ZR[hꏊ̈ܓxox擾B
	
	(in)
		adrCode   : ZR[h\

	(out)
		latitude   : ܓxixPʁj
		longitude  : oxixPʁj
	
	߂l
		0 : I
		 F	G[
				-1 : s 擾iKł̃G[
				-2 : 厚@@@@V
				-3 : ڎ@@@@V
				-4 : nԊX敄@@V
				-5 : }ԏZԍ@@V
				-6 : w肳ꂽZR[hɂ͈ܓxoxf[^
				
	
-----------------------------------------------------------------------
*/


int gcjAdrCode2Point( ADR_CODE *adrCode, 
                                    double *latitude, double *longitude )
{
	int nRet,nErr,codeLevel,secLat,secLon,jukyoHyoji;
	int cityCode,choCode,azaCode,jukyoCode,gaikuCode;
	GYOSEI_RECORD gyoseiRecord;
	CHO_RECORD choRecord;
	AZA_RECORD azaRecord;
	GAIKU_RECORD gaikuRecord;
	JUKYO_RECORD jukyoRecord;
	
	codeLevel = adrCode->codeLevel;
	if ( codeLevel < 1 ) { nErr = -1; goto err; }

	/* s惌R[h̎擾 */
	cityCode = adrCode->cityCode;
	
	nRet = GyoseiRecGet( cityCode, &gyoseiRecord );
	if ( nRet < 0 ) { nErr = -1; goto err; }
	
	if ( codeLevel <= KIND_CITY ) {
		secLat = gyoseiRecord.latitude;
		secLon = gyoseiRecord.longitude;
		if ( secLat == 0 || secLon == 0 ) 
			nRet = GetGaikuPoint( adrCode , &secLat, &secLon );
		goto ret;
	}

	/* 厚R[h̎擾 */
	choCode = adrCode->choCode;
	
	nRet = ChoRecGet( cityCode, choCode, &choRecord );
	if ( nRet < 0 ) { nErr = -2; goto err; }
	
	if ( codeLevel == KIND_CHO ) {
		secLat = choRecord.latitude;
		secLon = choRecord.longitude;
		if ( secLat == 0 || secLon == 0 ) 
			nRet = GetGaikuPoint( adrCode , &secLat, &secLon );
		goto ret;
	}

	/* ڎR[h̎擾 */
	azaCode = adrCode->azaCode;
	
	if ( azaCode ) {	/* ڎ͖ꍇ */
		nRet = AzaRecGet( cityCode, choCode, azaCode , &azaRecord );
		if ( nRet < 0 ) { nErr = -3; goto err; }
	
		if ( codeLevel == KIND_AZA ) {
			secLat = azaRecord.latitude;
			secLon = azaRecord.longitude;
			if ( secLat == 0 || secLon == 0 ) 
				nRet = GetGaikuPoint( adrCode , &secLat, &secLon );
			goto ret;
		}
	}

	/* nԊX惌R[h̎擾 */
	gaikuCode = adrCode->gaikuCode;
	
	nRet = GaikuRecGet( cityCode, choCode, azaCode, gaikuCode ,
										 &gaikuRecord, &jukyoHyoji );
	if ( nRet < 0 ) { nErr = -4; goto err; }

	if ( codeLevel == KIND_GAIKU ) {
		secLat = gaikuRecord.latitude;
		secLon = gaikuRecord.longitude;
		goto ret;
	}
	
	/* }ԏZԍR[h̎擾 */
	jukyoCode = adrCode->jukyoCode;
	
	nRet = JukyoRecGet( cityCode, choCode, azaCode, gaikuCode, 
											jukyoCode, &jukyoRecord );
	if ( nRet < 0 ) { nErr = -5; goto err; }
	
	if ( codeLevel == KIND_JUKYO ) {
		secLat = jukyoRecord.latitude;
		secLon = jukyoRecord.longitude;
		goto ret;
	}
	

ret:	
	*latitude = (double)secLat / 3600000;	/* ~bx */
	*longitude = (double)secLon / 3600000;
	
	return 0;

err:
	return nErr;
}



/* w肵ZR[h̋ɑX惌x̃f[^A̍ŏ̃R[h
@̈ܓxox擾B
@
@߂l@ 0FI
@@@@@@F擾łȂ
@
@EadrCode->codeLevelKIND_AZAȉiڎʃxj̏ꍇɎgpB
@@X惌xȉ͈ܓxoxĂ锤B
*/

static int GetGaikuPoint( ADR_CODE *adrCode , int *secLat, int *secLon )
{
	int codeLevel,retry,nret,jukyoHyoji;
	int cityCode,choCode,azaCode,gaikuCode,codeArray[2];
	ADR_CODE adrCode1;
	CHO_RECORD choRecord;
	GAIKU_RECORD gaikuRecord;
	
	codeLevel = adrCode->codeLevel;
	*secLat = *secLon = 0;

	cityCode = adrCode->cityCode;
	choCode = adrCode->choCode;
	azaCode = adrCode->azaCode;
	
	if ( codeLevel == KIND_CHO ) goto jcho;
	if ( codeLevel == KIND_AZA ) goto jaza;
/* s撬xȏサĂȂꍇ */
	/* 厚R[h̐ݒ */
	if ( codeLevel < KIND_CHO ){
		/* s撬R[h̐ݒ */
		switch ( codeLevel ){
			case KIND_KEN: cityCode = adrCode->cityCode + 101; break;

			case KIND_SHICHO: 
			case KIND_GUN: 
						cityCode = adrCode->cityCode + 1; break;
			default:		cityCode = adrCode->cityCode;

		}
		adrCode1.codeLevel = KIND_CHO;	/*@擾x */
		adrCode1.cityCode = cityCode;
		retry = 0;
j1:
		nret = gcjGetAdrRecordList( &adrCode1, codeArray, 2 );
		if ( nret < 2 ) {
			if ( retry ) return -1;
			adrCode1.cityCode = cityCode = cityCode + 100;	/* ʋAߎŝȂꍇ */
			retry = 1;
			goto j1;
		}
		choCode = codeArray[1];		/* ŏ̃R[h͖R[h */
	}
	else choCode = adrCode->choCode;

jcho:	/* 厚x܂Ŏ擾łĂꍇ */
	nret = ChoRecGet( cityCode ,choCode, &choRecord );
	if ( nret < 0 ) return -2;
	if ( choRecord.azaTableNum == 0 ) {
		azaCode = 0;
		goto jaza;
	}

	adrCode1.codeLevel = KIND_AZA;	/*@擾x */
	adrCode1.cityCode = cityCode;
	adrCode1.choCode = choCode;
	
	nret = gcjGetAdrRecordList( &adrCode1, &azaCode, 1 );
	if ( nret < 1 ) return -3;
	
jaza:	/* ڎx܂Ŏ擾łĂꍇ */
	adrCode1.codeLevel = KIND_GAIKU;	/*@擾x */
	adrCode1.cityCode = cityCode;
	adrCode1.choCode = choCode;
	adrCode1.azaCode = azaCode;
	
	nret = gcjGetAdrRecordList( &adrCode1, &gaikuCode, 1 );
	if ( nret < 1 ) return -4;
	
	/* Ẍܓxox擾 */
	nret = GaikuRecGet( cityCode, choCode, azaCode, gaikuCode, &gaikuRecord, &jukyoHyoji );
	if ( nret < 0 ) return -5;
	
	*secLat = gaikuRecord.latitude;
	*secLon = gaikuRecord.longitude;

	return 0;
}
