/*
* [gcjadrcode2str.c]
*
* Copyright (C) 2006 Orkney Inc., Kazuhide Kobayashi
*
* This program is free software; you can redistribute and/or modify
* it under the terms of the GNU Lesser General Public License
* as published by the Free Software Foundation.
*
* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
* GNU Lesser General Public License for more details.
*
* You should have received a copy of the GNU Lesser General Public
* License along with this program; if not, write to the Free Software
* Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307,
* USA.
*
* See the COPYING file for more information.
*
*
*/


/***********************************************************************

						Z擾B

************************************************************************
*/

#ifdef WIN32
#include <windows.h>
#endif

#include <stdio.h>
#include <string.h>
#include <malloc.h>

#include "geocoder.h"
#include "gcjlib.h"

#define NAME_MAX (KIND_MAGOBAN+1)

#define NUM_SEPARATOR "|"

typedef struct {
	char kanji[STRMAX+1];
	char kana[STRMAX+1];
} NAME_REC;

static NAME_REC name[ NAME_MAX ];

extern int g_encoding;
extern int g_outFormat;

static int SetGyoseiName( int gyoseiCode );

/*---------------------------------------------------------------------

	ZR[hꏊ̏Z擾B
	
	(in)
		adrCode   : ZR[h\
		kanjiBytes: Zi[obt@̃oCg
		kanaBytes : Z@@@@@@V
	(out)
		kanji     : Zij
		kana      : @@V@@ij
	
	߂l
		0ȏ : ϊłx
			     5 : }ԏZԍ܂
			     4 : nԊX敄܂
			     3 : ڎ܂
			     2 : 厚܂
			     1 : s於܂
			     0 : ϊłȂ
		 F	G[
				-1 : s於 擾iKł̃G[
				-2 : 厚@@@@V
				-3 : ڎ@@@@V
				-4 : nԊX敄@@V
				-5 : }ԏZԍ@@V
				-6 FR[hϊɂG[
				
	
	E܂͉̃obt@Ȃꍇ̏Kv

-----------------------------------------------------------------------
*/


int gcjAdrCode2Str( ADR_CODE *adrCode, char *kanji, int kanjiBytes,
												char *kana, int kanaBytes )
{
	int nRet,nErr,jukyoHyoji;
	int gyoseiCode,cityCode,choCode,azaCode,jukyoCode,gaikuCode;
	GYOSEI_RECORD gyoseiRecord;
	CHO_RECORD choRecord;
	AZA_RECORD azaRecord;
	GAIKU_RECORD gaikuRecord;
	JUKYO_RECORD jukyoRecord;
	char gaikuStr[STRMAX+1],jukyoStr[STRMAX+1];
	char *pKanji,*pKana,kanjiBuff[STRMAX+1],kanaBuff[STRMAX+1],kanaBuff2[STRMAX+1];
	int codeLevel,kind,nkanjiBytes,nkanaBytes,isSeireiShi;
	
	codeLevel = adrCode->codeLevel;
	memset( name, 0, sizeof( name ) );

	/* -----------------@s於̎擾 ---------------------- */
	/* s惌R[h̎擾 */
	if ( codeLevel < 1 ) { nErr = -1; goto err; }
	
	gyoseiCode = adrCode->cityCode;
	
	nRet = GyoseiRecGet( gyoseiCode, &gyoseiRecord );
	if ( nRet < 0 ) { nErr = -1; goto err; }
	
	kind = gyoseiRecord.shubetsu;

	/* ̊i[ */
	nRet = SetGyoseiName( gyoseiRecord.kenCode * 1000 );
	if ( nRet < 0 ) { nErr = -1; goto err; }
	if ( kind == KIND_KEN ) goto j1;

	/* x̊i[ */
		/* ͏o͂Ȃ */
	if ( ( gyoseiCode / 1000 ) == 13 && kind != KIND_SHICHO ) goto jgun;
	nRet = SetGyoseiName( gyoseiRecord.shichoCode );
	if ( nRet < 0 ) { nErr = -1; goto err; }
	if ( kind == KIND_SHICHO ) goto j1;

jgun:
	/* S̊i[ */
	nRet = SetGyoseiName( gyoseiRecord.gunCode );
	if ( nRet < 0 ) { nErr = -1; goto err; }
	isSeireiShi = ( gyoseiRecord.gunCode % 1000 ) < 200;
	if ( kind == KIND_GUN ) goto j1;

	/* s撬̊i[ */
	cityCode = gyoseiRecord.cityCode;
	nRet = SetGyoseiName( cityCode );
	if ( nRet < 0 ) { nErr = -1; goto err; }
	
	if ( codeLevel == KIND_CITY ) goto j1;

	/* -----------------@厚̎擾 ---------------------- */
	
	choCode = adrCode->choCode;
	
	nRet = ChoRecGet( cityCode, choCode, &choRecord );
	if ( nRet < 0 ) { nErr = -2; goto err; }
	
	gcjStrcpy( name[ KIND_CHO ].kanji, choRecord.name, STRMAX );
	gcjStrcpy( name[ KIND_CHO ].kana,  choRecord.kana, STRMAX );

	if ( codeLevel == KIND_CHO ) goto j1;

	/* -----------------@ڎ̎擾 ---------------------- */
	
	azaCode = adrCode->azaCode;
	
	if ( azaCode ) {	/* ڎ͖ꍇ */
		nRet = AzaRecGet( cityCode, choCode, azaCode , &azaRecord );
		if ( nRet < 0 ) { nErr = -3; goto err; }
	
		gcjStrcpy( name[ KIND_AZA ].kanji, azaRecord.name, STRMAX );
		gcjStrcpy( name[ KIND_AZA ].kana,  azaRecord.kana, STRMAX );
	}
	
	if ( codeLevel == KIND_AZA ) goto j1;

	/* -----------------@nԊX敄̎擾 ---------------------- */
	
	gaikuCode = adrCode->gaikuCode;
	
	nRet = GaikuRecGet( cityCode, choCode, azaCode, gaikuCode ,
										 &gaikuRecord, &jukyoHyoji );
	if ( nRet < 0 ) { nErr = -4; goto err; }
/*@!!!!!!! nԕ̕ϊs !!!!!!!*/
	sprintf( gaikuStr, "%d", gaikuRecord.gaikuCode );\
	gcjStrcpy( name[ KIND_GAIKU ].kanji, gaikuStr, STRMAX );
	
	if ( codeLevel == KIND_GAIKU ) goto j1;
	
	/* -----------------@}ԏZԍ̎擾 ---------------------- */
	
	jukyoCode = adrCode->jukyoCode;
	
	nRet = JukyoRecGet( cityCode, choCode, azaCode, gaikuCode, 
											jukyoCode, &jukyoRecord );
	if ( nRet < 0 ) { nErr = -5; goto err; }
	
/*@!!!!!!! nԕ̕ϊs !!!!!!!*/
	sprintf( jukyoStr, "%d", jukyoRecord.jukyoCode );
	gcjStrcpy( name[ KIND_JUKYO ].kanji, jukyoStr, STRMAX );
	
	if ( codeLevel == KIND_JUKYO ) goto j1;
	

j1:	

	pKana = kanaBuff;
	nkanaBytes = STRMAX;
	if ( g_encoding ){
		pKanji = kanjiBuff;
		nkanjiBytes = STRMAX;
	}
	else {
		pKanji = kanji;
		nkanjiBytes = kanjiBytes;
	}
	
	/* ̃Rs[ */
	if ( g_outFormat & OUT_KEN ){
		gcjStrcpy( pKanji, name[ KIND_KEN ].kanji, nkanjiBytes );
		gcjStrcpy( pKana, name[ KIND_KEN ].kana, nkanaBytes );
	}
	
	/* x̃Rs[ */
	if ( g_outFormat & OUT_SHICHO ){
		gcjStrcat( pKanji, name[ KIND_SHICHO ].kanji, nkanjiBytes );
		gcjStrcat( pKana, name[ KIND_SHICHO ].kana, nkanaBytes );
	}
	
	/* Sߎs̃Rs[ */
	if ( ( (! isSeireiShi) && (g_outFormat & OUT_GUN) ) || 
			( isSeireiShi && ( g_outFormat & OUT_SEIREI) ) ){
		gcjStrcat( pKanji, name[ KIND_GUN ].kanji, nkanjiBytes );
		gcjStrcat( pKana, name[ KIND_GUN ].kana, nkanaBytes );
	}
	
	/* s撬̃Rs[ */
	if ( g_outFormat & OUT_CITY ){
		gcjStrcat( pKanji, name[ KIND_CITY ].kanji, nkanjiBytes );
		gcjStrcat( pKana, name[ KIND_CITY ].kana, nkanaBytes );
	}
	
	/* 厚̃Rs[ */
	if ( g_outFormat & OUT_CHO ){
		gcjStrcat( pKanji, name[ KIND_CHO ].kanji, nkanjiBytes );
		gcjStrcat( pKana, name[ KIND_CHO ].kana, nkanaBytes );
	}
	
	/* ڎ̃Rs[ */
	if ( g_outFormat & OUT_AZA ){
		char *pChome,*pAza;
		pAza = name[ KIND_AZA ].kanji;
		
		/* ڂ̏ꍇ */
		pChome = strstr( pAza, "" );
		if ( pChome ){
			char buff[STRMAX+1];
				/* Zpŏo͂ꍇ */
			if ( ( g_outFormat & OUT_CHOME_KANJI ) == 0 ){
				int n,matchLength;
				
				n = NumStr2Int( pAza, &matchLength );
				if ( matchLength ){
					Int2ZenkakuStr( n, buff, STRMAX );
					gcjStrcat( pKanji, buff, nkanjiBytes );
					gcjStrcat( pKana, buff, nkanaBytes );
				}
				else goto j2;
			}
				/* ̏ꍇ */
			else {
				char c = *pChome;
				*pChome = '\0';
				gcjStrcat( pKanji, pAza, nkanjiBytes );
				gcjStrcat( pKana, pAza, nkanaBytes );
				*pChome = c;
			}
			/* ""̏o */
			if ( g_outFormat & OUT_CHOME_STR ){
				gcjStrcat( pKanji, "", nkanjiBytes );
				gcjStrcat( pKana, "", nkanaBytes );
			}
		}
		/* ̏ꍇ */
		else {
j2:
			gcjStrcat( pKanji, name[ KIND_AZA ].kanji, nkanjiBytes );
			gcjStrcat( pKana, name[ KIND_AZA ].kana, nkanaBytes );
		}
	}

	/* IncPЂ̃f[^ɊւĂ͏Z\̏񂪖̂ŁAnԊX敄̒l
	@ŔfibdlƂj */
	if ( gaikuCode >= 100 ) jukyoHyoji = 0;
	
	/* nԊX敄̃Rs[ */
	if ( g_outFormat & OUT_GAIKU ){
		if ( strlen( name[ KIND_GAIKU ].kanji )) {
				/*ڂŏIĂ|t */
			if ( LastCharIsNumeric( pKanji ) ) {
						gcjStrcat( pKanji, NUM_SEPARATOR, nkanjiBytes );
						gcjStrcat( pKana, NUM_SEPARATOR, nkanaBytes );
			}
				/* nԊX敄 */
			gcjStrcat( pKanji, name[ KIND_GAIKU ].kanji, nkanjiBytes );
			gcjStrcat( pKana, name[ KIND_GAIKU ].kana, nkanaBytes );
				/* ԁAԒn */
			if ( g_outFormat & OUT_BANCHI_STR ){
				if ( jukyoHyoji ) gcjStrcat( pKanji, "", nkanjiBytes );
				else gcjStrcat( pKanji, "Ԓn", nkanjiBytes );
			}
		}
	}
	
	/* }ԏZԍ̃Rs[ */
	if ( g_outFormat & OUT_JUKYO ){
		if ( strlen( name[ KIND_JUKYO ].kanji )) {
				/*nԊX敄ŏIĂ|t */
			if ( LastCharIsNumeric( pKanji )) {
						gcjStrcat( pKanji, NUM_SEPARATOR, nkanjiBytes );
						gcjStrcat( pKana, NUM_SEPARATOR, nkanaBytes );
			}
				/* }ԏZԍ */
			gcjStrcat( pKanji, name[ KIND_JUKYO ].kanji, nkanjiBytes );
			gcjStrcat( pKana, name[ KIND_JUKYO ].kana, nkanaBytes );
				/*  */
			if ( g_outFormat & OUT_BANCHI_STR ){
				if ( jukyoHyoji ) gcjStrcat( pKanji, "", nkanjiBytes );
			}
		}
	}

	/* pJiSpЂ炪Ȃɕϊ */
	*kanaBuff2 = '\0';
	gcjKanaHan2Zen( pKana, kanaBuff2, STRMAX );
	
	/* R[h̕ϊ */
	if ( g_encoding ){
		int mode;

		if ( g_encoding == EUCJP ) mode = SJIS2EUCJP;
		else if ( g_encoding == UTF8 ) mode = SJIS2UTF8;
		else return -6;
		
		/* ̂̕ϊ */
		nRet = gcjCharConv( mode, pKanji, kanji, kanjiBytes );
		if ( nRet < 0 ) return -6;

		/* Ȗ̂̕ϊ */
		nRet = gcjCharConv( mode, kanaBuff2, kana, kanaBytes );
		if ( nRet < 0 ) return -6;
	}
	else gcjStrcpy( kana, kanaBuff2, kanaBytes );
	
	return codeLevel;
err:
	strcpy( kanji, "" );
	strcpy( kana, "" );
	return nErr;
}


/*
	sR[h̎sname[]Ɋi[B
	
	߂l
		0 : IigyoseiCode0̎܂ށj
		-1 : s惌R[h擾G[
*/

static int SetGyoseiName( int gyoseiCode )
{
	int nRet;
	GYOSEI_RECORD gyoseiRecord;
	
	if ( gyoseiCode == 0 ) return 0;
	
	/* QR̍s͖ */
	if ( gyoseiCode == 13100 ){
		strcpy( name[ KIND_GUN ].kanji, "" );
		strcpy( name[ KIND_GUN ].kana, "" );
		return 0;
	}
	
	nRet = GyoseiRecGet( gyoseiCode, &gyoseiRecord );
	if ( nRet != 0 ) return -1;

	switch( gyoseiRecord.shubetsu ){
		case KIND_KEN :
			gcjStrcpy( name[ KIND_KEN ].kanji, gyoseiRecord.name, STRMAX );
			gcjStrcpy( name[ KIND_KEN ].kana,  gyoseiRecord.kana, STRMAX );
			break;
			
		case KIND_SHICHO :
			gcjStrcpy( name[ KIND_SHICHO ].kanji, gyoseiRecord.name, STRMAX );
			gcjStrcpy( name[ KIND_SHICHO ].kana,  gyoseiRecord.kana, STRMAX );
			
			break;

		case KIND_GUN :
			gcjStrcpy( name[ KIND_GUN ].kanji, gyoseiRecord.name, STRMAX );
			gcjStrcpy( name[ KIND_GUN ].kana,  gyoseiRecord.kana, STRMAX );
			break;

		case KIND_CITY:
			gcjStrcpy( name[ KIND_CITY ].kanji, gyoseiRecord.name, STRMAX );
			gcjStrcpy( name[ KIND_CITY ].kana,  gyoseiRecord.kana, STRMAX );
			break;
			
		default:
			return -2;
	}
	return 0;
}

