/*
* [gcjcharconv.c]
*
* Copyright (C) 2006 Orkney Inc., Kazuhide Kobayashi
*
* This program is free software; you can redistribute and/or modify
* it under the terms of the GNU Lesser General Public License
* as published by the Free Software Foundation.
*
* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
* GNU Lesser General Public License for more details.
*
* You should have received a copy of the GNU Lesser General Public
* License along with this program; if not, write to the Free Software
* Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307,
* USA.
*
* See the COPYING file for more information.
*
*
*/

/***********************************************************************

	gcjCharConv Function : R[hϊ֐

************************************************************************
*/

#ifdef WIN32
#include <windows.h>
#endif

#include <stdio.h>
#include <string.h>
#include <malloc.h>

#include "geocoder.h"
#include "gcjlib.h"

#define STR_SJIS "SJIS-win"
#define STR_EUCJP "EUC-JP-MS"
#define STR_UTF8 "UTF8"

#ifdef WIN32
int ConvSjisUtf8( int mode, char* fromStr, char *toStr, int toStrBytes );
int Sjis2Eucjp( char* fromStr, char *toStr, int toStrBytes );
int Eucjp2Sjis( char* fromStr, char *toStr, int toStrBytes );
#else
#include <iconv.h>
#include <errno.h>
#endif

/*---------------------------------------------------------------------

	R[hϊB
		
	(in)
		mode : R[hϊ@
			0 : SJIS2EUCJP  1: EUCJP2SJIS
			2 : SJIS2UTF8   3: UTF82SJIS
		fromStr : ϊ镶
		numBytes : ϊ̕i[obt@̗e
	(out)
		toStr : ϊ̕
	
	߂l
		0 : I
		F	G[
			-1 : o̓obt@̗eʂȂ
			-2ȉ : ̑̕ϊ̃G[
			
-----------------------------------------------------------------------
*/


int gcjCharConv( int mode, char *fromStr, char *toStr, int numBytes )
{
#ifdef WIN32
	int nret;
	
	switch ( mode ){
		case SJIS2EUCJP:
			nret = Sjis2Eucjp( fromStr, toStr, numBytes );
			break;
			
		case EUCJP2SJIS:
			nret = Eucjp2Sjis( fromStr, toStr, numBytes );
			break;
		
		case SJIS2UTF8:
			nret = ConvSjisUtf8( 1, fromStr, toStr, numBytes );
			break;
		
		case UTF82SJIS:
			nret = ConvSjisUtf8( 0, fromStr, toStr, numBytes );
			break;
	}
	if ( nret >= 0 ) return 0;
	if ( nret == -1 ) return -1;
	else return -2;

#else
/* for Linux */

	int inBytesLeft,outBytesLeft,nret;
	iconv_t ic;
	char *fromCode,*toCode;

	switch ( mode ){
		case SJIS2EUCJP:
			fromCode = STR_SJIS;
			toCode = STR_EUCJP;
			break;
			
		case EUCJP2SJIS:
			fromCode = STR_EUCJP;
			toCode = STR_SJIS;
			break;
		
		case SJIS2UTF8:
			fromCode = STR_SJIS;
			toCode = STR_UTF8;
			break;
		
		case UTF82SJIS:
			fromCode = STR_UTF8;
			toCode = STR_SJIS;
			break;
	}
	
	ic = iconv_open( toCode, fromCode );
	if ( ic == (iconv_t)-1 ) return -2;

	inBytesLeft = strlen( fromStr );
	outBytesLeft = numBytes;
	nret = iconv( ic, &fromStr, &inBytesLeft, &toStr, &outBytesLeft );
	if ( outBytesLeft > 1 ){
		*toStr = '\0';
		*toStr = '\0';
	}
		
	iconv_close(ic);

	if ( nret < 0 ){
		if ( errno == E2BIG) return -1;
		else if ( errno == EILSEQ || errno == EINVAL) return -3;
		else return -4;
	}
	
	return 0;

#endif

}


#ifdef WIN32

/******************************************************************************
                       Windowsp̕R[hϊ֐
-------------------------------------------------------------------------------
*/


/*
	VtgJISUTF8̕Ԃ̕ϊsB
	
	(in)
		mode    : 0 = UTF8VtgJISɁ@ 1 = VtgJISUTF8ɕϊ
		fromStr : ϊ̕
		toStrBytes: ϊobt@̃oCg
		
	(out)
		toStr   : ϊ̕obt@
	
	߂l=	i[oCg
			FG[
			   -1 : ϊ̃obt@̗eʂȂ
			   -2ȉF̑̃G[
*/

int ConvSjisUtf8( int mode, char* fromStr, char *toStr, int toStrBytes )
{
	WCHAR *unicodeBuff;
	int nret,fromCode,toCode,numWchar;

	if ( mode == 0 ) {
		fromCode = CP_UTF8;
		toCode = CP_ACP;
	}
	else {
		fromCode = CP_ACP;
		toCode = CP_UTF8;
	}

	/* UNICODEi[obt@̊m */	
	numWchar = MultiByteToWideChar( fromCode, 0, fromStr, -1, NULL, 0 );
	if ( numWchar == 0 ) return -2;
	
	unicodeBuff = (WCHAR*) malloc( numWchar * 2 );
	if ( ! unicodeBuff  ) return -3;
	
	/* UNICODEɕϊ */
	nret = MultiByteToWideChar( fromCode, 0, fromStr, -1, unicodeBuff, numWchar );
	if ( nret == 0 ) { nret = -4; goto ret; }
	
	/* VtgJIS܂UTF8ɕϊ */
	nret = WideCharToMultiByte( toCode, 0, unicodeBuff , numWchar, toStr, toStrBytes, NULL, NULL );
	if ( nret == 0 ) { 
		nret = GetLastError();
		if ( nret == ERROR_INSUFFICIENT_BUFFER ) nret = -1;
		else nret = -5;
	}
	else nret = 0;

ret:
	if ( unicodeBuff ) free( unicodeBuff );
	return nret;
}


/*
	VtgJIS̕EUC-JɕϊB
	
	(in)
		fromStr : ϊ̕
		toStrBytes: ϊobt@̃oCg
		
	(out)
		toStr   : ϊ̕obt@
	
	߂l=	i[oCg
			FG[
			   -1 : ϊ̃obt@̗eʂȂ
*/

int Sjis2Eucjp( char* fromStr, char *toStr, int toStrBytes )
{
	unsigned char c1,c2,d1,d2;
	int remains = toStrBytes;
	
	while(1){
		c1 = (unsigned char) *fromStr++;
		if ( c1 == '\0' )break;

			/* Sp̏ꍇ */
		if ( ( c1 >= 0x81 && c1 <= 0x9f ) || ( c1 >= 0xe0 && c1 <= 0xef ) ){
			c2 = (unsigned char) *fromStr++;
			if ( c2 == '\0' ) break;

			if ( c2 < 0x9f ){	/* QoCg@0x40 - 0x9e */

				if ( c1 < 0xa0 ) d1 = ( c1 - 0x81 ) * 2 + 0xa1;
				else d1 = ( c1 - 0xe0 ) * 2 + 0xdf;

				if ( c2 > 0x7f ) d2 = c2 + 0x60;
				else d2 = c2 + 0x61;
			}
			else {	/* QoCg@0x9f - 0xfc */

				if ( c1 < 0xa0 ) d1 = ( c1 - 0x81 ) * 2 + 0xa2;
				else d1 = ( c1 - 0xe0 ) * 2 + 0xe0;

				d2 = c2 + 2;
			}
			if ( remains-- ) *toStr++ = d1;
			else break;
			
			if ( remains-- ) *toStr++ = d2;
			else break;
		}
		else if ( c1 >= 0xa1 && c1 <= 0xdf ){	/* pJȉꍇ */
			if ( remains-- ) *toStr++ = (char) 0x8e;
			else break;
			
			if ( remains-- ) *toStr++ = c1;
			else break;
		}
		else {
			if ( remains-- ) *toStr++ = c1;
			else break;
		}
	}
	if ( remains ) {
		*toStr = '\0';
		return toStrBytes - remains;
	}
	else return -1;
}

/*
	EUC-JP̕VtgJISɕϊB
	
	(in)
		fromStr : ϊ̕
		toStrBytes: ϊobt@̃oCg
		
	(out)
		toStr   : ϊ̕obt@
	
	߂l=	i[oCg
			FG[
			   -1 : ϊ̃obt@̗eʂȂ
	
	EEUC-JPJIS X0212̕܂܂ꍇAϊ
	@ɂ́Hi[ĂB
*/

int Eucjp2Sjis( char* fromStr, char *toStr, int toStrBytes )
{
	unsigned char c1,c2,d1,d2;
	int remains = toStrBytes;
	
	while(1){
		c1 = (unsigned char) *fromStr++;
		if ( c1 == '\0' )break;

		if  ( c1 >= 0xa1 && c1 <= 0xfe ) { /* Sp̏ꍇ */
			c2 = (unsigned char) *fromStr++;
			if ( c2 == '\0' ) break;

			/* PoCg̕ϊ */
			if ( c1 < 0xdf ) d1 = ( c1 + 1 ) / 2 + 0x30;
			else d1 = ( c1 + 1 ) / 2 + 0x70;

			/* QoCg̕ϊ */
			if ( ( c1 % 2 ) == 0 ) d2 = c2 - 2;
			else {
				if ( c2 < 0xe0 ) d2 = c2 - 0x61;
				else d2 = c2 - 0x60;
			}
			if ( remains-- ) *toStr++ = d1;
			else break;
			
			if ( remains-- ) *toStr++ = d2;
			else break;
		}
		else if ( c1 == 0x8e ){	/* pJȉꍇ */
			c2 = (unsigned char) *fromStr++;
			if ( c2 == '\0' ) break;

			if ( remains-- ) *toStr++ = c2;
			else break;
		}
		else if ( c1 == 0x8f ){	/* JIS X0212̏ꍇ */
			char *p = "H";	/* ֕ */
			
			/* ƂQoCg|C^i߂BvRoCg */
			if ( *fromStr++ == '\0' ) break;
			if ( *fromStr++ == '\0' ) break;

			/* VtgJISɂ͑֕i[ */
			if ( remains-- ) *toStr++ = *p++;
			else break;
			if ( remains-- ) *toStr++ = *p;
			else break;
		}
		else {
			if ( remains-- ) *toStr++ = c1;
			else break;
		}
	}
	if ( remains ) {
		*toStr = '\0';
		return toStrBytes - remains;
	}
	else return -1;
}

#endif

