/*
* [gyoseihash.c]
*
* Copyright (C) 2006 Orkney Inc., Kazuhide Kobayashi
*
* This program is free software; you can redistribute and/or modify
* it under the terms of the GNU Lesser General Public License
* as published by the Free Software Foundation.
*
* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
* GNU Lesser General Public License for more details.
*
* You should have received a copy of the GNU Lesser General Public
* License along with this program; if not, write to the Free Software
* Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307,
* USA.
*
* See the COPYING file for more information.
*
*
*/

/***********************************************************************

	GyoseiHash Function : s於nbVe[u֐

************************************************************************
*/

#ifdef WIN32
#include <windows.h>
#endif

#include <stdio.h>
#include <string.h>
#include <malloc.h>

#include "geocoder.h"
#include "gcjlib.h"

/*-------------------------
	ϐ
---------------------------
*/

	/* s於nbVe[u */
static GYOSEI_HASH_HEADER m_gyoseiHashHeader;
static int *m_gyoseiHashTable;
static GYOSEI_HASH_BUCKET *m_aryBucket;

static int m_numRecords;
static int m_hashSize;

/*---------------------------------------------------------------------

	s於nbVe[ut@C̓eɊi[B
	
	(in)
		fileName : s於nbVe[ut@C̖
	
	E͂̊֐ŊmۂB
	ȄIAGyoseiHashEnd()ĂяoŃJB
	
	߂l
		0 : 	I
		F	G[
			-1 : nbVpmۂłȂ
			-2 : t@CI[vłȂ
			-3 : t@CID̕񂪈Ⴄ
			-4 .. -8 : t@C̃[hG[
			
-----------------------------------------------------------------------
*/

int GyoseiHashLoad( char *fileName )
{
	FILE *fpHash;
	int n,nRet;
	
	fpHash = fopen( fileName, "rb" );
	if ( fpHash == NULL ){ nRet = -2; goto err; }
	
		/* wb_[ */
	n = fread( &m_gyoseiHashHeader, sizeof( m_gyoseiHashHeader ), 1, fpHash);
	if ( n != 1 ){ nRet = -4; goto err; }

	if ( strcmp( m_gyoseiHashHeader.fileId, FILE_ID_GYOSEIHASH ) ) { nRet = -3; goto err; }
	m_numRecords = m_gyoseiHashHeader.numRecords;
	m_hashSize = m_gyoseiHashHeader.hashSize;
	
		/* nbVe[u */
	m_gyoseiHashTable = (int*) malloc( m_hashSize * sizeof(int) );
	if ( ! m_gyoseiHashTable ){ nRet = -1; goto err; }
	
	n = fread( m_gyoseiHashTable, sizeof(int), m_hashSize, fpHash);
	if ( n != m_hashSize ){ nRet = -5; goto err; }

		/* oPbg */
	m_aryBucket = (GYOSEI_HASH_BUCKET*) malloc( sizeof(GYOSEI_HASH_BUCKET) * m_numRecords );
	if ( ! m_aryBucket ){ nRet = -1; goto err; }

	n = fread( m_aryBucket, sizeof(GYOSEI_HASH_BUCKET), m_numRecords, fpHash );
	if ( n != m_numRecords ) { nRet = -6; goto err; }
	
	fclose( fpHash );
	return 0;

err:
	if ( m_aryBucket ) {
		free( m_aryBucket );
		m_aryBucket = NULL;
	}
	if ( m_gyoseiHashTable ) {
		free( m_gyoseiHashTable );
		m_gyoseiHashTable = NULL;
	}
	fclose( fpHash );
	return nRet;
}


/*---------------------------------------------------------------------

	s於nbVe[ȕIs
	
	EmۂJB
	
	߂l
		0 : 	I
		
-----------------------------------------------------------------------
*/

int GyoseiHashEnd()
{
	if ( m_aryBucket ) {
		free( m_aryBucket );
		m_aryBucket = NULL;
	}
	if ( m_gyoseiHashTable ) {
		free( m_gyoseiHashTable );
		m_gyoseiHashTable = NULL;
	}
	return 0;
}


/*---------------------------------------------------------------------

	s於nbVe[ugpāAZ̐擪ɂs於
	B
	
	(in)
		str        : Z
		maxArray   : codeArray̍ővf
	(out)
		matchLength : }b`O񕔕̃oCg
		shubetsu    : s̎ʁiPFs{@2Fx@3:Sߎs@
		@@@@@@@@@@@@@@@@@@@@@@@@@@4:s撬j
		codeArray : sR[h
		  
	߂l
		 : i[sR[h̐
		   0 : Ȃ
		F G[
			-1 : nbVe[uɃ[hĂȂ
			-2 : Z̒O
			-3 : ̑ʏN肦ȂG[

	EZDBɂs於̂ŁAZ̐擪ɍv̂TB

	Es於͏Z̐擪matchLength̃oCg
	@łB
	@̐̕ȕ\LGyoseiRecGet()֐ɂm鎖łB

	EZɍvsŕꍇAő̒̂̂ԂB
		ijZ񂪁hRsh̏ꍇAhhƁhRs"
		      ̕ōv邪A蒷Rs̃R[ĥ݂ԂB

-----------------------------------------------------------------------
*/

int GyoseiHashSearch( char *str, int *matchLength, int *shubetsu,
												int *codeArray, int maxArray )
{
	int i,numFound,hash,maxLength,index;
	long bucketNum;
	char *p;
	int foundCode[100][3]; /* [0]=sR[h@[1]= [2]= */

	if ( ! m_gyoseiHashTable ) return -1;
	if ( ! strlen( str ) ) return -2;
	
	/* w蕶̓Q̃nbVľvZ */
	hash = (*(unsigned long*)(str)) % m_hashSize;

	/* nbVe[uoPbgԍ擾 */
	bucketNum = m_gyoseiHashTable[hash];
	numFound = 0;	
	
	/* oPbg̒strƈv̂擾 */
	if ( bucketNum ){	/* Qv̂ */
		while (1){
			 p = strstr( str, m_aryBucket[bucketNum].name );
			if ( p ){
				if ( p == str ){	/* ̍ŏłΌ */
					/* }b`ő咷̂̂I邽߂Ɉꎞۑ */
					foundCode[ numFound ][0] = m_aryBucket[bucketNum].gyoseiCode;
					foundCode[ numFound ][1] = strlen( m_aryBucket[bucketNum].name );
					foundCode[ numFound++ ][2] = m_aryBucket[bucketNum].shubetsu;
					if ( numFound == 100 ) return -3;
				}
			}
			bucketNum = m_aryBucket[bucketNum].nextBucketNum ;
			if ( bucketNum ) continue;
			else break;
		}
		if ( ! numFound ) return 0;

		if ( numFound == 1  ) maxLength = foundCode[0][1];
		else {		/* ꍇ͍ő咷̂̂T */
			maxLength = 0;
			for ( i = 0; i < numFound; i++ )
				if ( foundCode[i][1] > maxLength ) maxLength = foundCode[i][1];
		}
		
		/* ő咷̂̂Ԃ */
		index = 0;
		for ( i = 0; i < numFound; i++ ){
			if ( (foundCode[i][1]) != maxLength ) continue;
			codeArray[index++] = foundCode[i][0];
			*shubetsu = foundCode[i][2];
			if ( index == maxArray ) break;
		}
		*matchLength = maxLength;
		return index;
	}
	return 0;
}
