/*
* [mojidx.c]
*
* Copyright (C) 2006 Orkney Inc., Kazuhide Kobayashi
*
* This program is free software; you can redistribute and/or modify
* it under the terms of the GNU Lesser General Public License
* as published by the Free Software Foundation.
*
* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
* GNU Lesser General Public License for more details.
*
* You should have received a copy of the GNU Lesser General Public
* License along with this program; if not, write to the Free Software
* Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307,
* USA.
*
* See the COPYING file for more information.
*
*
*/

/***********************************************************************

	MojiIndex Function : CfbNX֐

************************************************************************
*/

#ifdef WIN32
#include <windows.h>
#endif

#include <stdio.h>
#include <string.h>
#include <malloc.h>

#include "geocoder.h"
#include "gcjlib.h"

#define TABLE_MAX 65536

static MOJI_INDEX_HEADER m_mojiIndexHeader;
static MOJI_TABLE_DATA m_mojiTableList[ TABLE_MAX ];
static int *m_indexTable;
static SOURCE_RECORD *m_sourceRecTable;

static int m_numIndexRecords;	/* SẴ̕CfbNXR[hie[uŌ0܂ށj */
static int m_numSourceRecords; /* R[h */

static int SearchSameNumbers( int *set1, int num1, int *set2, int num2, int *set3, int max3 );
static int binarySearch( int n, int *ary, int *startPos, int endPos );

/*---------------------------------------------------------------------

	CfbNXt@C̓eɊi[B
	
	(in)
		fileName : CfbNXt@C̖
	
	E͂̊֐ŊmۂB
	ȄIAMojiIndexEnd()ĂяoŃJB
	
	߂l
		0ȏ : i[R[h
		F	G[
			-1 : mۂłȂ
			-2 : t@CI[vłȂ
			-3 : t@CID̕񂪈Ⴄ
			-4ȍ~ : t@C̃[hG[
			
-----------------------------------------------------------------------
*/

int MojiIndexLoad( char *fileName )
{
	FILE *fpRec;
	int n,nRet;
	
	fpRec = fopen( fileName, "rb" );
	if ( fpRec == NULL ) return -2;
	
	/* wb_[ */
	n = fread( &m_mojiIndexHeader, sizeof( m_mojiIndexHeader ), 1, fpRec );
	if ( n != 1 ) { nRet = -4; goto err; }

	if ( strcmp( m_mojiIndexHeader.fileId, FILE_ID_MOJI_INDEX ) ) { nRet = -3; goto err; }
	m_numIndexRecords = m_mojiIndexHeader.numIndexRecords;
	m_numSourceRecords = m_mojiIndexHeader.numSourceRecords;
	
	/* CfbNXe[uXg */
	n = fread( &m_mojiTableList, sizeof( m_mojiTableList ), 1, fpRec );
	if ( n != 1 ) { nRet = -5; goto err; }

	/* CfbNXe[u */	
	m_indexTable = (int*) malloc( sizeof(int) * m_numIndexRecords );
	if ( ! m_indexTable ){ nRet = -1; goto err; }

	n = fread( m_indexTable, sizeof(int), m_numIndexRecords, fpRec );
	if ( n != m_numIndexRecords ) { nRet = -6; goto err; }
	
	/* R[he[u */	
	m_sourceRecTable = (SOURCE_RECORD*) malloc( sizeof(SOURCE_RECORD) * m_numSourceRecords );
	if ( ! m_sourceRecTable ){ nRet = -1; goto err; }

	n = fread( m_sourceRecTable, sizeof(SOURCE_RECORD), m_numSourceRecords, fpRec );
	if ( n != m_numSourceRecords ) { nRet = -6; goto err; }

	fclose( fpRec );
	return m_numIndexRecords;

err:
	if ( m_indexTable ) {
		free( m_indexTable );
		m_indexTable = NULL;
	}
	if ( m_sourceRecTable ) {
		free( m_sourceRecTable );
		m_sourceRecTable = NULL;
	}
	fclose( fpRec );
	return nRet;
}


/*---------------------------------------------------------------------

	CfbNX̏Is
	
	EmۂJB
	
	߂l
		0 : 	I
		
-----------------------------------------------------------------------
*/

int MojiIndexEnd()
{
	if ( m_indexTable ) {
		free( m_indexTable );
		m_indexTable = NULL;
	}
	if ( m_sourceRecTable ) {
		free( m_sourceRecTable );
		m_sourceRecTable = NULL;
	}
	
	return 0;
}

/*---------------------------------------------------------------------

	CfbNXgāAZnB
	
	(in)
		str        : n
		maxArray   : codeArray̍ővf
	(out)
		codeArray : R[h\R[h̔z
		  
	߂l
		 : i[nR[h̐
		   0 : Ȃ
		F G[
			-1 : CfbNXe[uɃ[hĂȂ
			-2 : n̒O
			-3 : codeArray̍ővf܂Ŋi[B
			-4 : n̒iő势ROj
			-5ȉF̑̃G[

	EʂƂē͎̂̃f[^łB
	@Pjw肳ꂽ̊eSĊ܂ށB
	@QjAAw肳ꂽ̏̕AʒuƂ͈Ⴄ̂܂ށB

	Ew肳ꂽ񂾂܂ނ̂~ꍇ́Aꂽ
	@strstr()ŃtB^OB

	EcodeArray NULL̏ꍇ́AR[ĥݕԂB
	
-----------------------------------------------------------------------
*/

#define MAX_CHARS 30
#define REC_MAX 125000

typedef struct {
	int numRecords;
	int *indexTable;
} INDEX_TABLE_DATA;

static int buff1[ REC_MAX ],buff2[ REC_MAX ];
	
int MojiIndexSearch( char *str, SOURCE_RECORD *codeArray, int maxArray )
{
	INDEX_TABLE_DATA indexTableData[ MAX_CHARS ];
	MOJI_TABLE_DATA *pTableData;
	int *pIndex[ MAX_CHARS ],numRecords[ MAX_CHARS ];
	int *set1,num1,*set3;
	char *p;
	int i,n,charCode,numMatchChars,nret,recNum;
	
	if ( strlen( str ) == 0 ) return -2;
	
	/* eɑ΂CfbNX擾 */
	p = str;
	i = 0;
	while( *p ){
		/* R[ȟvZ */
			/* ̏ꍇ͂QoCgŕR[hvZ */
		if ( IsSjisKanji((char*)p) ) {
			charCode = *( unsigned short*) p;
			p += 2;
		}
			/* p͖ */
		else {
			p++;
			continue;
		}
			
		/*@R[hɑΉCfbNXe[ũe[uf[^擾 */
		pTableData = & m_mojiTableList[ charCode ];
		indexTableData[i].numRecords = pTableData->numRecords;
		indexTableData[i].indexTable = m_indexTable + pTableData->tableTopPos;
		
			/* e[u̍Ō̃R[hԍ0mF */
		if ( *( indexTableData[i].indexTable + indexTableData[i].numRecords - 1 ) != 0 )
						return -5;

		i++;
		if ( i == MAX_CHARS ) return -4;
	}
	numMatchChars = i;	/* }b`ȎΏە */
	if ( numMatchChars == 0 ) return 0;

	/* eCfbNX̃R[h|C^̐ݒ*/
	for ( i = 0; i < numMatchChars; i++ ) {
		pIndex[i] = indexTableData[i].indexTable;
		numRecords[i] = indexTableData[i].numRecords - 1;	/* e[uŌ0͏ */
	}

	/* P̏ꍇ́ACfbNX̃R[ĥ܂ܕԂ */
	if ( numMatchChars == 1 ){
		if ( codeArray == NULL ) return numRecords[0];
		
		if ( numRecords[0] > maxArray ) {
			nret = -3;
			n = maxArray;
		}
		else {
			nret = n = numRecords[0];
		}
		for ( i = 0; i < n; i++ ){
			recNum = *( pIndex[0] + i);
			if ( recNum < 1 || recNum > m_numSourceRecords ) return -6;
			codeArray[i] = m_sourceRecTable[ recNum ];
		}
		return nret;
	}
		
	/* ẽCfbNXɂAR[hԍ{ */
	set1 = pIndex[0];
	num1 = numRecords[0];
	set3 = buff1;
	for ( i = 1; i < numMatchChars; i++ ){
		nret = SearchSameNumbers( set1, num1, pIndex[i], numRecords[i], set3, REC_MAX );
		if ( nret == -1 ) return -7;	/* v */
		if ( nret == 0 ) return 0;		/* vȂ */
		
		if ( set3 == buff1 ) {
			set1 = buff1;
			set3 = buff2;
		}
		else {
			set1 = buff2;
			set3 = buff1;
		}
		num1 = nret;
	}
	
	/*@ʂi[ */
	if ( codeArray == NULL ) return num1;
	
	if ( num1 < maxArray ) {
		nret = n = num1;
	}
	else {
		n = maxArray;
		nret = -3;
	}
	
	for ( i = 0; i < num1; i++ ){
		codeArray[i] = m_sourceRecTable[ set1[i] ];
	}
	return num1;
}

/*
	Q̐̏WŁAǂɂ݂lB
	set1[]  : Pڂ̏W
	num1  : @@V@@@̗vf
	set2[]  : Qڂ̏W
	num2  : @@V@@@̗vf
	set3[]  : ʂԂobt@
	max3  : @@V@@@@@@̍ővf
	
	߂l=@0ȏF@obt@Ɋi[vf
	@@@@-1    :  obt@̗eʌx܂Ŋi[
*/

static int SearchSameNumbers( int *set1, int num1, int *set2, int num2, int *set3, int max3 )
{
	int i,*p1,*p2,n1,n2,startPos,numFound,n,nret;
	
	/* vf̏Ȃɂ */
	if ( num1 < num2 ){
		p1 = set1;
		n1 = num1;
		p2 = set2;
		n2 = num2;
	}
	else {
		p1 = set2;
		n1 = num2;
		p2 = set1;
		n2 = num1;
	}
	
	/*@W1̗vfɂāAW2ɑ݂邩ׂB
	@@ׂʂgāAW2͈̌͂΂߂ĂB
	*/
	startPos = 0;
	numFound = 0;
	for ( i = 0; i < n1; i++ ){
		n = p1[i];
		nret = binarySearch( n, p2, &startPos, n2 - 1 );
		if ( nret == -1 ) break;	/* z̍Ō܂Ō */
		if ( nret == 1 ){	/*  */
			*set3++ = n;
			numFound++;
			if ( numFound == max3 ) return -1;
		}
	}
	return numFound;
}

/*@oCiT[`s
	(in)
	n : {l
	ary : ̔z
	startPos : TJnz̈ʒu
	endPos   : TIz̈ʒu
	
	(out)
	startPos : n傫l̂ŏ̐̈ʒu
	
	߂l=@1 : l݂
	@@@@ 0 : @@@@݂Ȃ
	@@@@ -1: z̍Ō܂ő݂Ȃ

	EnƓlꍇ͂𒴂łlB
*/

static int binarySearch( int n, int *ary, int *startPos, int endPos )
{
	int i,found,index,cmp;
	int upperIndex,lowerIndex;

	if ( *startPos > endPos ) return -1;
	
	found = 0;	

	upperIndex = endPos;
	lowerIndex = *startPos;

	while(1){
		index = ( upperIndex + lowerIndex ) / 2;
		if ( ( cmp = n - *(ary + index) ) == 0 ) {
			found = 1;
			break;
		}
		else if (cmp < 0 ) upperIndex = index - 1;
		else lowerIndex = index + 1;

		if ( lowerIndex > upperIndex ) break;
	}		

	if (found) {
		/* n傫Ȓl̈ʒu߂ */
		for ( i = index + 1; i <= endPos; i++ ){
			if ( n < *(ary + i) ) break;
		}
		*startPos = i;
		return 1;
	}
	else {
		*startPos = lowerIndex;
		if ( lowerIndex > endPos ) return -1;
		return 0;
	}
}
