/*
* [util.c]
*
* Copyright (C) 2006 Orkney Inc., Kazuhide Kobayashi
*
* This program is free software; you can redistribute and/or modify
* it under the terms of the GNU Lesser General Public License
* as published by the Free Software Foundation.
*
* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
* GNU Lesser General Public License for more details.
*
* You should have received a copy of the GNU Lesser General Public
* License along with this program; if not, write to the Free Software
* Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307,
* USA.
*
* See the COPYING file for more information.
*
*
*/

/***********************************************************************

     	gcj Function : e탆[eBeB֐

************************************************************************
*/

#ifdef WIN32
#include <windows.h>
#endif

#include <stdio.h>
#include <string.h>
#include <ctype.h>

#include "geocoder.h"
#include "gcjlib.h"

#define STRMAX 1023
typedef unsigned short USHORT;

/*-------------------------
	ϐ
---------------------------
*/

static USHORT m_convChars[ 65536 ];
static int m_convCharsLoaded;


/*-------------------------
	֐
---------------------------
*/
static int InitConvChars();

int IsSjisKanji( char *p );




/* ߎwss̋於̍sR[h23Aߎwss̍sR[h
@擾B
@ijF14131@@sF14130@擾B
@
@߂l 
		Fߎwss̍sR[h
		FG[
		     -1: ߎwss̃R[hł͂Ȃ

	E̓s{ɂQȏ̐ߎwssꍇAR[h
	@30ÂƂŕϊĂB
	@s140ԑȂ̂ŕʏĂB
*/

int gcjGetSeireiCode( int gyoseiCode )
{
	int kuCode,kenCode;
	
	kuCode = gyoseiCode % 1000;
	kenCode = gyoseiCode / 1000;

	if ( kuCode < 100 || kuCode > 199 ) return -1;

	if ( kenCode == 27 ) {	/* { */
		if ( kuCode >= 140 && kuCode < 160 ) return 27400;
	}
	kuCode -= 100;
	return kenCode * 1000 + ( kuCode / 30 ) * 30 + 100;
			
}



/* overflowNȂ strcpy()
	destBytes : destŊmۂĂoCgi\0̕܂ރoCgj
	
	EdestBytes𒴂ꍇ́Ax܂ŃRs['\0'ŏI[B
	
	߂l = 0:I
	        -1:Sɂ̓Rs[łȂ
	        -2:łȂ
*/

int gcjStrcpy( char *dest, char *source, int destBytes )
{
	if ( destBytes <= 0 || dest == NULL || source == NULL ) return -2;
	
	if ( strlen( source ) < (unsigned int)destBytes - 1 ){
		strcpy( dest, source );
		return 0;
	}

	strncpy( dest, source, destBytes - 1 );
	*( dest + destBytes - 1 ) = '\0';
	return -1;
}
		
/* overflowNȂ strcat()
	destBytes : destŊmۂĂoCgi\0̕܂ރoCgj
	
	EdestBytes𒴂ꍇ́Ax܂ŃRs[B
	
	߂l = 0:I
	        -1:Sɂ̓Rs[łȂ
	        -2:łȂ
*/

int gcjStrcat( char *dest, char *source, int destBytes )
{
	int n;
	
	if ( destBytes <= 0 || dest == NULL || source == NULL ) return -2;
	
	if ( strlen( dest ) + strlen( source ) < (unsigned int)destBytes - 1 ){
		strcat( dest, source );
		return 0;
	}
	n = destBytes - strlen( dest ) - 1;
	if ( n < 0 ) return -1;
	
	strncat( dest, source, n );
	*( dest + destBytes - 1 ) = '\0';
	return -1;
}


/*
	𐔒lɕϊB
	
	E͊,SpZp,pZp
	@̏ꍇ͂PłȂƂȂB
	
	EȊO̕ĂƂ̒̕O܂ł̐lϊB
	
	EPȂ"0"͖߂l0ɂȂ邪ALȐ񂪖ꍇ0Ŗ
	@̂ŋʂtȂB
	
	
*/

int NumStr2Int( char *str, int *matchLength )
{
	char *pStr;
	int num,subTotal,total,n;
	
	if ( ! m_convCharsLoaded ) InitConvChars();
	
	pStr = str;
	num = 0;
	subTotal = 0;
	total = 0;
	while( *pStr ){
		if ( IsSjisKanji( pStr ) ) {
			n = m_convChars[ *(USHORT *) pStr ];
			if ( n == 0xffff ) break;
			pStr += 2;
		}
		else {
			if ( isdigit( *pStr ) ) n = *pStr++ - '0';
			else break;
		}
			
		if ( n < 10 )  num = num * 10 + n;
		else if ( n >= 10 && n < 10000){
			if ( num == 0 && ( n == 10 || n == 100 || n == 1000) ) num = 1;
			subTotal += num * n;
			num = 0;
		}
		else if ( n >= 10000 ){
			total += ( subTotal + num ) * n;
			subTotal = 0;
			num = 0;
		}
	}
	total += subTotal + num;
	if ( matchLength ) *matchLength = pStr - str;
	return total;
}



/*@ϊpzB
	ϊe
	PD͔p
	QDSpZp͔p
	
*/

static int InitConvChars()
{
	char kansuji[18] = { "OlܘZ" };
	char suji[20] = { "OPQRSTUVWX" };
	int i;

	if ( m_convCharsLoaded ) return 0;
	
	memset( m_convChars, 0xff, sizeof(m_convChars) );
	for ( i = 0; i < 9; i++ ) m_convChars[ *(USHORT *)(kansuji+i*2) ] = i + 1;
	for ( i = 0; i < 10; i++ ) m_convChars[ *(USHORT *)(suji+i*2) ] = i ;
#ifdef WIN32
	m_convChars[ *(USHORT *)"\" ] = 10;
#else
	m_convChars[ *(USHORT *)"\\ " ] = 10;
#endif
	m_convChars[ *(USHORT *)"S" ] = 100;
	m_convChars[ *(USHORT *)"" ] = 1000;
	m_convChars[ *(USHORT *)"" ] = 10000;
	m_convChars[ *(USHORT *)"" ] = 10000;

	m_convCharsLoaded = 1;

	return 0;
}

/* ̍Ō̕ǂׂ
*/

int LastCharIsNumeric( char *str )
{
	char *p;
	int n,isNumeric;
	
	if ( ! m_convCharsLoaded ) InitConvChars();

	isNumeric = 0;
	p = str + strlen( str ) - 2;	/* ŌォQ */
	if ( IsSjisKanji( p ) ) {
		n = m_convChars[ *(USHORT *) p ];
		isNumeric = ( n != 0xffff );
	}
	else isNumeric = isdigit( *(p+1) );
	
	return isNumeric;
}


/* SpZpԂ

	߂l=@0 : I
	@@@@-1 : obt@̗eʂȂ
*/

int Int2ZenkakuStr( int n, char *str, int strBytes )
{
	char buff[STRMAX+1];
	int i,numDigits;
	
	sprintf( buff, "%d", n );
	numDigits = strlen( buff );
	if ( numDigits * 2 + 1 > strBytes ) return -1;
	
	for ( i = 0; i < numDigits; i++ ){
		*str++ = (char) 0x82;
		*str++ = 0x4f + buff[i] - '0';
	}
	*str = '\0';
	
	return 0;
}

/* VtgJISǂׂ

	߂l
	@@@ 1 : VtgJISł
	@@@ 0 : VtgJISłȂiPoCgڂQoCgڂ͈͂ɓĂȂj
*/

int IsSjisKanji( char *p )
{
	unsigned char c;
	
	c = *(unsigned char *)p++;
	if ( (0x81 <= c && c <= 0x9f) || (0xe0 <= c && c <= 0xfc) ){
		c = *(unsigned char *)p;
		if ( 0x40 <= c && c <= 0xfc && c != 0x7f ) return 1;
	}
	return 0;
}


/*
	pJiSpЂ炪ȂɕϊB
	
	߂l=@i[oCg
	@@@@-1Fi[̃oCgȂB
	@@@@
	ELނ̓T|[gĂȂ
*/

#define NUM_HIRA_CHARS 116
int gcjKanaHan2Zen( char *hankaku, char *zenkaku, int zenkakuMax )
{
	char hiragana[NUM_HIRA_CHARS] = {   "[ĂƂȂɂʂ˂̂͂Ђӂւق܂݂ނ߂JK" };
	char dakuten[NUM_HIRA_CHARS] = {    "################################Âł##########΂тԂׂ##################################" };
	char hanDakuten[NUM_HIRA_CHARS] = { "########################################################################ς҂Ղ؂##################################" };
	char *pKana,*pZenkaku;
	int index;

	if ( strlen( hankaku ) * 2 + 1 > (unsigned int )zenkakuMax ) return -1;
	
	pZenkaku = zenkaku;
	while( *hankaku ){
		index = ( *hankaku - '' ) * 2;

		/* pJiłȂ */
		if ( index >= NUM_HIRA_CHARS || index < 0 ){ 
			if ( IsSjisKanji(hankaku) ) *pZenkaku++ = *hankaku++;
			*pZenkaku++ = *hankaku++;
			continue;
		}
		
		/* ̕_̏ꍇ */
		if ( (*(hankaku+1) == '') && (dakuten[ index ] != '#') ) pKana = dakuten;

		/* ̕_̏ꍇ */
		else if ( (*(hankaku+1) == '') && (hanDakuten[ index ] != '#') ) pKana = hanDakuten;
		else pKana = hiragana;
				
		*pZenkaku++ = pKana[ index ];
		*pZenkaku++ = pKana[ index + 1 ];
		hankaku++;
		if ( pKana != hiragana ) hankaku++;
	}
	*pZenkaku = '\0';
	return pZenkaku - zenkaku;	
}

/*
	SpЂ炪Ȃ𔼊pJ^JiɕϊB
	
	߂l=@i[oCg
	@@@@-1Fi[̃oCgȂB
	@@@@
	ELނ̓T|[gĂȂ
*/

#define NUM_KANA_CHARS 83
int gcjKanaZen2Han( char *zenkaku, char *hankaku, int hankakuMax )
{
	char *pHankaku,*pHankakuMax;
	unsigned char *pZenkaku;
	int index,c;
	char katakana[ NUM_KANA_CHARS ] = { "޷޸޹޺޻޼޽޾޿ޯӬԭծܲ" };

	pHankaku = hankaku;
	pHankakuMax = hankaku + hankakuMax - 1;
	pZenkaku = (unsigned char*) zenkaku;
	while( *pZenkaku ){
		if ( pHankaku == pHankakuMax ){ *pHankaku = '\0'; return -1; }
		
		/* PƂ̑_A_Ȁꍇ */
		if ( *pZenkaku == 0x81 ){
			c = 0;
			switch ( *( pZenkaku + 1 ) ){
				case 0x4a: c = ''; break;
				case 0x4b: c = ''; break;

				case 0x5b:
				case 0x5c:
				case 0x5d:
				case 0x60: c = ''; break;
			}
			if ( c != 0 ) {
				*pHankaku++ = c;
				pZenkaku += 2;
				continue;
			}
		}
		
		if ( *pZenkaku == 0x82 ) index = *(pZenkaku+1) - 0x9f;
		else index = -1;

		/* Ђ炪ȂłȂ͂̂܂܃Rs[ */
		if ( index >= NUM_KANA_CHARS || index < 0 ){ 
			if ( IsSjisKanji(pZenkaku) ) {
				*pHankaku++ = *pZenkaku++;
				if ( pHankaku == pHankakuMax ){ *pHankaku = '\0'; return -1; }
			}
			*pHankaku++ = *pZenkaku++;
			continue;
		}
		
		/* ̏ꍇ */
		if ( katakana[ index ] == '' ) {
			c = '';
			index--;
		}
		/* ̏ꍇ */
		else if ( katakana[ index ] == '' ) {
			c = '';
			index -= 2;
		}
		else c = 0;

		*pHankaku++ = katakana[ index ];
		if ( pHankaku == pHankakuMax ){ *pHankaku = '\0'; return -1; }
		if ( c != 0 ) *pHankaku++ = c;
		pZenkaku += 2;
	}
	*pHankaku = '\0';
	return pHankaku - hankaku;
}




/*
	strpbrk()̃VtgJIS
		str[]   : Ώۂ̕
		chars[] : 镶Wi1000ȓ)
		
	߂l
		ŏɌstr[]̏ꏊւ̃|C^
		NULL@Ȃ
		
		
*/
#define CMAX 1000

char *sjispbrk( char *str, char *chars )
{

	char *p,*pMax;
	int i,numChars,nStr,nChars[CMAX],np;
	
	numChars = strlen( chars ) / 2;
	if ( numChars > CMAX ) return NULL;
	p = chars;
	for ( i = 0; i < numChars; i++ ) {
		nChars[i] = *(short*)p ;
		p += 2;
	}

	p = chars;
	i = 0;
	while ( *p ){
		if ( i == CMAX ) break;
		if ( IsSjisKanji(p) ){
			nChars[i++] = *(short*)p ;
			p += 2;
		}
		else{
			nChars[i++] = *p;
			p++;
		}
	}
	numChars = i;
	
	p = str;
	pMax = str + strlen(str);
	while( *p ){
		nStr = *(short*)p;

		if ( IsSjisKanji(p) ){
			nStr= *(short*)p ;
			np = 2;
		}
		else{
			nStr = *p;
			np = 1;
		}

		for ( i = 0; i < numChars; i++ ){
			if ( nStr == nChars[i] ) break;
		}
		if ( i < numChars ) break;
		p += np;
	}
	if( *p ) return p;
	else return NULL;
}


		
