/*
* [geocoderja.h]
*
* Copyright (C) 2006 Orkney Inc., Kazuhide Kobayashi
*
* This program is free software; you can redistribute and/or modify
* it under the terms of the GNU Lesser General Public License
* as published by the Free Software Foundation.
*
* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
* GNU Lesser General Public License for more details.
*
* You should have received a copy of the GNU Lesser General Public
* License along with this program; if not, write to the Free Software
* Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307,
* USA.
*
* See the COPYING file for more information.
*
*
*/

/*
	GeoCoder Japan : GCJ - function header file 
*/

#ifndef _GEOCODERJP_H
#define _GEOCODERJP_H

/* obt@̍ő啶 */
#define STRMAX 1023

/* R[h */
#define SJIS 0
#define EUCJP  1
#define UTF8 2

#define SJIS2EUCJP 0
#define EUCJP2SJIS 1
#define SJIS2UTF8 2
#define UTF82SJIS 3

/* ZpDB̎ނу[h郌x*/
#define DB_AUTO      0
#define DB_GYOSEI    1
#define DB_CHO       2
#define DB_AZA       3
#define DB_GAIKU     4
#define DB_JUKYO     5

#define DB_CHIMEI25K 14
#define DB_MOJI_INDEX 16

/*  Z̎ */
#define KIND_KEN        1
#define KIND_SHICHO     2
#define KIND_GUN        3
#define KIND_CITY       4
#define KIND_CHO        5
#define KIND_AZA        6
#define KIND_GAIKU      7
#define KIND_JUKYO      8
#define KIND_MAGOBAN    9


/*  Zn̎ */
#define NAME_KEN        1
#define NAME_SHICHO     2
#define NAME_GUN        3
#define NAME_CITY       4
#define NAME_CHO        5
#define NAME_AZA        6
#define NAME_CHIMEI25K  10


typedef struct {
    int codeLevel;    /* R[hLȃx@*/
    int cityCode;     /* s撬R[h(JIS) 5 @@@FxS@*/
    int choCode;      /* E厚R[hiƎj    @@FxT@*/
    int azaCode;      /* R[hiƎj     @@@ FxU@*/
    int gaikuCode;    /* nԁAX敄R[hiƎj@FxV@*/
    int jukyoCode;    /* nԂ̎}ԁAZԍ@   @@ FxW@*/
} ADR_CODE ;


#define CHIMEI25K_NAME_LEN 47

typedef struct {
	int chimei25kCode;/* 25000nR[h */
	int cityCode;     /* s撬R[h(JIS) 5 */
	int kind;         /* 1FRn 2FZn 3Fyn̗pi */
    char name[CHIMEI25K_NAME_LEN+1]; /* n */
	double latitude;  /* ܓxixPʁj */
	double longitude;    /* oxi@V@j */
} CHIMEI25K_DATA;


#ifdef _cplusplus_
extern "C" {
#endif

/* R[hɊւ֐ */
int gcjSetArgEncoding( char *encoding );
int gcjCharConv( int mode, char *fromStr, char *toStr, int numBytes );

/* ZDBɊւ֐ */
int gcjDbSetPath( char *dbDirPath );
int gcjDbLoad( char *dbDirPath, int loadLevel );
int gcjDbEnd();

/* ZɊւ֐ */
int gcjAdrStr2Code( char *address, ADR_CODE *adrCode );
int gcjAdrCode2Str( ADR_CODE *adrCode, char *kanji, int kanjiBytes,
							char *kana, int kanaBytes );
int gcjAdrCode2Point( ADR_CODE *adrCode, 
                                    double *latitude, double *longitude );
int gcjPost2AdrCode( int postCode, ADR_CODE *adrCode );
int gcjGetNumAdrRecords( ADR_CODE *adrCode );
int gcjGetAdrRecordList( ADR_CODE *adrCode, int *codeArray,
									                  int recordMax );
int gcjGetAdrRecord( ADR_CODE *adrCode, void* record, int recordBytes );

/* 25000nɊւ֐ */
int gcjChimei25kSearch( char *chimeiStr, CHIMEI25K_DATA *chimei25kData, int maxData );

/* ̑ */
int gcjKanaHan2Zen( char *hankaku, char *zenkaku, int zenkakuMax );
int gcjKanaZen2Han( char *zenkaku, char *hankaku, int hankakuMax );
int gcjAdrFilter( int filterNum, char *fromStr, char *toStr, int toStrMax );


#ifdef _cplusplus_
}
#endif

#endif

