/*
* [linematch.c]
*
* Copyright (C) 2006 Orkney Inc., Kazuhide Kobayashi
*
* This program is free software; you can redistribute and/or modify
* it under the terms of the GNU Lesser General Public License
* as published by the Free Software Foundation.
*
* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
* GNU Lesser General Public License for more details.
*
* You should have received a copy of the GNU Lesser General Public
* License along with this program; if not, write to the Free Software
* Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307,
* USA.
*
* See the COPYING file for more information.
*
*
*/



/*
     Geocoder sample program

     usage : linematch gcjDbDir encoding
				gcjDbDir = geocoder database directory
				encoding = 0: sjis(default)  1: euc-jp  2: utf8

	function:
		Read address string from stdin.
		Output matching result to stdout in the following format.
		"level=(matching level) lat=(latitude in degree) lon=(longitude) str=(converted address)"
*/

#ifdef WIN32
#define _cplusplus_
#include "stdafx.h"
#include "windows.h"
#else
#include <iconv.h>
#endif

#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include "geocoderja.h"

int main(int argc, char* argv[])
{
	int nret,matchLength,matchLevel,charCode,error;
	char *p,buffer[STRMAX+1],kanji[STRMAX+1],kana[STRMAX+1],*pCharCode;
	char buffer2[STRMAX+1];
	double latitude,longitude;
	ADR_CODE adrCode;

	if ( argc < 2 ){
		printf("usage : linematch gcjDbPath charCode\n");
		return 0;
	}

	printf( "Now loading\n" );
	nret = gcjDbLoad( argv[1], DB_AUTO );
	if ( nret < 0 ){
		printf("Can't load address database nret=%d\n",nret );
		return 0;
	}

	if ( argc == 3 ) {
		charCode = atoi( argv[2] );
		if ( charCode > 0 ){
			if ( charCode == 1 ) pCharCode = "EUC-JP";
			else pCharCode = "UTF-8";
			nret = gcjSetArgEncoding( pCharCode );
			if ( nret < 0 ) {
				printf( "Set encoding erro : nret= %d\n", nret );
				return 0;
			}
		}
	}
	
	printf( "Please input address string.\n" );
	while( !feof( stdin ) ){
		if ( ! fgets( buffer, STRMAX, stdin )) break;
		p = strchr( buffer, '\n' );
		if ( p ) *p = '\0';

	
		gcjAdrFilter( 1, buffer, buffer2, STRMAX );
		matchLength = gcjAdrStr2Code( buffer2, &adrCode );
		if ( matchLength >0 ){
			matchLevel = gcjAdrCode2Str( &adrCode, kanji, STRMAX, kana, STRMAX );
			if ( matchLevel > 0 ){
				nret = gcjAdrCode2Point( &adrCode, &latitude, &longitude );
				if ( nret == 0 ){
					printf( "level=%d lat=%.6lf lon=%.6lf str=%s\n",
											matchLevel, latitude, longitude, kanji );
					error = 0;
				}
				else printf( "matched but error when convert to point.\n" );
			}
			else printf( "matched but error when convert to string.\n" );
		}
		else if ( matchLength == -1 ) printf( "more than one match.\n" );
		else printf( "no match.\n" );
	}
	gcjDbEnd();

	return 0;

}

