/*
* [match.c]
*
* Copyright (C) 2006 Orkney Inc., Kazuhide Kobayashi
*
* This program is free software; you can redistribute and/or modify
* it under the terms of the GNU Lesser General Public License
* as published by the Free Software Foundation.
*
* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
* GNU Lesser General Public License for more details.
*
* You should have received a copy of the GNU Lesser General Public
* License along with this program; if not, write to the Free Software
* Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307,
* USA.
*
* See the COPYING file for more information.
*
*
*/



/*
	Geocoder sample program

	usage : match inputName outputName gcjDbPath encoding

		inputName   : address list file name ( full path )
		outputName : output file name ( full path )
		gcjDbPath  : geocder database directory
		encoding   : 0 : SJIS(default)  1: EUC-JP  2: UTF8

	function:
		Read address from inut file.
		Output geocoding result to output file.
*/


#ifdef WIN32
#define _cplusplus_

#include "stdafx.h"
#include "windows.h"
#else
#include <sys/time.h>
#include <iconv.h>
int GetTickCount();
#include <errno.h>
#endif

#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include "geocoder.h"
#include "gcjlib.h"

int main(int argc, char* argv[])
{
	FILE *fpList, *fpOut;
	int nret,matchLength,matchLevel,count,startTime,endTime,charCode;
	char *p,buffer[STRMAX+1],outBuff[STRMAX+1],kanji[STRMAX+1],kana[STRMAX+1];
	double latitude,longitude;
	ADR_CODE adrCode;
	char *pCharCode;

	fpList = fpOut = NULL;
	charCode = 0;
	
	if ( argc < 4 ){
		printf("match inputFile outputFile gcjDbPath encoding\n");
		return 0;
	}
		
	
	fpList = fopen( argv[1], "r" );
	if ( !fpList ){
		printf("Can't open the file : %s\n", argv[1] );
		return 0;
	}
	
	fpOut = fopen( argv[2], "w" );
	if ( !fpOut ){
		printf("Can't create the file : %s\n", argv[2] );
		goto ret;
	}

	nret = gcjDbLoad( argv[3], DB_AUTO );
	if ( nret < 0 ){
		printf("Can't load address database nret=%d\n",nret );
		goto ret;
	}

	if ( argc == 5 ) {
		charCode = atoi( argv[4] );
		if ( charCode > 0 ){
			if ( charCode == 1 ) pCharCode = "EUC-JP";
			else pCharCode = "UTF-8";
			nret = gcjSetArgEncoding( pCharCode );
			if ( nret < 0 ) {
				printf( "Set encoding erro : nret= %d\n", nret );
				return 0;
			}
		}
	}
	
	startTime = GetTickCount();
	count = 0;
	while( !feof( fpList ) ){
		
		p = fgets( buffer, STRMAX, fpList );
		if ( !p ) break;
		
		count++;
		nret = fputs( buffer, fpOut );
		if ( nret < 0 ){
			printf("Write error.\n" );
			goto ret;
		}
		
		char buffer2[STRMAX+1];
		gcjAdrFilter( 1, buffer, buffer2, STRMAX );
		matchLength = gcjAdrStr2Code( buffer2, &adrCode );

/*		matchLength = gcjAdrStr2Code( buffer, &adrCode );*/

		if ( matchLength == 0 ) {
			sprintf( outBuff, "    > no match\n" );
		}
		else if ( matchLength < 0 ){
			sprintf( outBuff, "    > get code err = %d\n", matchLength );
		}
		else{
			matchLevel = gcjAdrCode2Str( &adrCode, kanji, STRMAX, kana, STRMAX );
			if ( matchLevel < 0 ){
				sprintf( outBuff, "    > gcjAdrCode2Str() error = %d\n", matchLevel );
			}
			else {
				nret = gcjAdrCode2Point( &adrCode, &latitude, &longitude );
				if ( nret < 0 ){
					sprintf( outBuff, "    > gcjAdrCode2Point() error = %d\n", nret );
				}
				else {
					sprintf( outBuff, "    > level= %d  lat = %.6lf lon = %.6lf\n     %s\n",
											matchLevel, latitude, longitude, kanji );
				}
			}
		}
		strcat( outBuff, "\n" );
		
		nret = fputs( outBuff, fpOut );
		if ( nret < 0 ){
			printf("Write error.\n" );
			goto ret;
		}
	}
	endTime = GetTickCount();
	sprintf( outBuff, "\n\n Number of Lines = %d\n\nProcessing time = %d (msec)\n",
									count, endTime - startTime );
	nret = fputs( outBuff, fpOut );
		

ret:
	if ( fpList ) fclose( fpList );
	if ( fpOut ) fclose( fpOut );
	gcjDbEnd();
		
	return 0;

}

#ifndef WIN32
// for linux
int GetTickCount()
{
        struct timeval tv;
        struct timezone tz;

        gettimeofday( &tv, &tz );
        return tv.tv_sec * 1000 + tv.tv_usec/1000;
}



int sJis2Utf8( char *fromStr, char *toStr, int numBytes )
{
        int inBytesLeft,outBytesLeft,ret;
        iconv_t ic;

        ic = iconv_open( "UTF8", "SJIS" );
        if ( ic == (iconv_t)-1 ) return -1;

        inBytesLeft = strlen( fromStr );
        outBytesLeft = numBytes;
        ret = iconv( ic, &fromStr, &inBytesLeft, &toStr, &outBytesLeft );
        if ( ret < 0 ){
                if ( errno == EILSEQ) printf("EILSEQ\n");
                if ( errno == EINVAL) printf("EINVAL\n");
                if ( errno == E2BIG) printf("E2BIG\n");
        }

        iconv_close(ic);

        if ( ret < 0 ) return -1;
        else return 0;
}
#endif

